from typing import Dict, List, Optional, Union, Callable, Coroutine, Type
from rekuest.api.schema import WidgetInput, PortGroupInput
from rekuest.definition.registry import get_default_definition_registry
from rekuest.structures.registry import StructureRegistry
from rekuest.structures.default import get_default_structure_registry
from rekuest.actors.types import Actifier
import os
from functools import wraps


def create_arkitekt_folder(with_cache: bool = True):
    """Create the arkitekt folder"""
    os.makedirs(".arkitekt", exist_ok=True)
    if with_cache:
        os.makedirs(".arkitekt/cache", exist_ok=True)

    gitignore = os.path.join(".arkitekt", ".gitignore")
    dockerignore = os.path.join(".arkitekt", ".dockerignore")
    if not os.path.exists(gitignore):
        with open(gitignore, "w") as f:
            f.write(
                "# Hiding Arkitekt Credential files from git\n*.json\n*.temp\ncache/"
            )
    if not os.path.exists(dockerignore):
        with open(dockerignore, "w") as f:
            f.write(
                "# Hiding Arkitekt Credential files from git\n*.json\n*.temp\ncache/"
            )

    return ".arkitekt"





def dockerbuild(**kwargs):

    """Deploy to a docker container.

    Args:

        **kwargs: Keyword arguments passed to the docker client.

    Returns:

        A docker container object.

    """

    return {"type": "docker", "image": "arkitekt/deployers:latest"}


from rich.console import Group
from rich.tree import Tree
from rich.panel import Panel
from arkitekt.apps import Arkitekt
from typing import MutableSet, Tuple, Any, Dict
import os


def construct_changes_group(changes: MutableSet[Tuple[Any, str]]) -> Group:
    panel_header = "Detected changes\n"

    actor_tree = Tree("Changes in", style="white not bold")
    panel_group = Group(panel_header, actor_tree)
    for change, path in changes:
        actor_tree.add(os.path.normpath(path))

    return panel_group


def construct_app_group(app: Arkitekt) -> Group:
    panel_header = f"Running App \n\n{app.identifier}:{app.version}\n"

    actor_tree = Tree("Registered Definitions", style="white not bold")
    panel_group = Group(panel_header, actor_tree)
    for key, definition in app.rekuest.agent.definition_registry.definitions.items():
        actor_tree.add(key)

    return panel_group


def construct_leaking_group(variables: Dict[str, Any]) -> Group:
    panel_header = (
        "Detected leaking Variables\n\n"
        "[white]Your app is leaking variables. "
        "Leaking variables can cause memory leaks and other issues."
        "Please make sure you are not defining variables in the"
        "global scope (outside of functions). "
        "If you want to define constants please make them all UPPERCASE\n\n"
    )

    actor_tree = Tree("Leaking Variables", style="white not bold")
    panel_group = Group(panel_header, actor_tree)
    for key, value in variables.items():
        actor_tree.add("key: " + key + " value: " + str(value))

    return panel_group


def construct_run_panel(app: Arkitekt) -> Panel:
    app_group = construct_app_group(app)

    return Panel(
        app_group, style="bold green", border_style="green", title="Arkitekt Run"
    )
